import { baseDecode, baseEncode } from 'borsh';
declare class Call {
    origin_public_key: Uint8Array;
    receipt_public_key: Uint8Array;
    encrypted_egress: boolean;
    transaction_action: number;
    amount: number | null;
    contract_name: Uint8Array | null;
    method: Uint8Array | null;
    args: Uint8Array | null;
    wasm_code: Uint8Array | null;
    constructor(config: {
        origin_public_key: Uint8Array;
        receipt_public_key: Uint8Array;
        encrypted_egress: boolean;
        transaction_action: number;
        amount: number | null;
        contract_name: Uint8Array | null;
        method: Uint8Array | null;
        args: Uint8Array | null;
        wasm_code: Uint8Array | null;
    });
}
declare class Calls {
    ops: Call[];
    shard_id: number;
    block_number: number | null;
    constructor(config: {
        ops: Call[];
        shard_id: number;
        block_number: number | null;
    });
}
declare class Outcome {
    view_result_log: Uint8Array[];
    view_result: Uint8Array | null;
    view_error: Uint8Array | null;
    outcome_logs: Uint8Array[];
    outcome_token_burnt: number;
    outcome_status: Uint8Array | null;
    encrypted: Uint8Array | null;
    constructor(config: {
        view_result_log: Uint8Array[];
        view_result: Uint8Array | null;
        view_error: Uint8Array | null;
        outcome_logs: Uint8Array[];
        outcome_token_burnt: number;
        outcome_status: Uint8Array | null;
        encrypted: Uint8Array | null;
    });
}
declare class Outcomes {
    ops: Outcome[];
    call_index: number;
    signature: Uint8Array;
    state_root: Uint8Array;
    constructor(config: {
        ops: Outcome[];
        call_index: number;
        signature: Uint8Array;
        state_root: Uint8Array;
    });
}
declare const callSchema: Map<typeof Call, {
    kind: string;
    fields: ((string | (string | number)[])[] | (string | {
        kind: string;
        type: string;
    })[] | (string | {
        kind: string;
        type: string[];
    })[])[];
}>;
declare const callsSchema: Map<any, any>;
declare const outcomeSchema: Map<typeof Outcome, {
    kind: string;
    fields: ((string | string[][])[] | (string | {
        kind: string;
        type: string[];
    })[])[];
}>;
declare const outcomesSchema: Map<any, any>;
declare const buildCall: (call: Call) => Uint8Array;
declare const buildCalls: (calls: Calls) => Uint8Array;
declare const buildOutcome: (outcome: Outcome) => Uint8Array;
declare const buildOutcomes: (outcomes: Outcomes) => Uint8Array;
declare const parseCall: (buf: Uint8Array) => Call;
declare const parseCalls: (buf: Uint8Array) => Calls;
declare const parseOutcome: (buf: Uint8Array) => Outcome;
declare const parseOutcomes: (buf: Uint8Array) => Outcomes;
export { Call, callSchema, buildCall, parseCall, Calls, callsSchema, buildCalls, parseCalls, Outcome, outcomeSchema, buildOutcome, parseOutcome, Outcomes, outcomesSchema, buildOutcomes, parseOutcomes, baseEncode, baseDecode };
